package controller;

import javafx.animation.KeyFrame;
import javafx.animation.Timeline;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.ToggleButton;
import javafx.util.Duration;
import model.FirefighterBoard;
import view.FirefighterGrid;

public class Controller {

  private FirefighterBoard board;
  private Timeline timeline;

  @FXML
  private Label generationNumberLabel;

  @FXML
  private Button restartButton;

  @FXML
  private Button oneStepButton;

  @FXML
  private ToggleButton playToggleButton;

  @FXML
  private ToggleButton pauseToggleButton;

  @FXML
  private FirefighterGrid grid;

  public void initialize(int boxWidth, int boxHeight, int columns, int rows, int fireCount, int firefighterCount, int initialRoadCount,int initialcloudCount, int initialMountainsCount, int initialRocksCount, int initialFiretruckCount) {
    // Initialize the simulation board and connect it to the grid
    board = new FirefighterBoard(columns, rows, fireCount, firefighterCount, initialRoadCount,initialcloudCount, initialMountainsCount, initialRocksCount, initialFiretruckCount);
    grid.initialize(columns, rows, boxWidth, boxHeight, board);

    // Initialize timeline for automatic progression
    timeline = new Timeline(new KeyFrame(Duration.millis(100), event -> updateGame()));
    timeline.setCycleCount(Timeline.INDEFINITE);

    // Ensure initial UI state is consistent
    updateGenerationLabel();
    pauseToggleButton.setSelected(true);
    stopSimulation();
    board.reset();
    grid.repaint();
    updateGenerationLabel();
  }


  @FXML
  private void restartButtonAction() {
    stopSimulation();
    board.reset();
    grid.repaint();
    updateGenerationLabel();
  }
  @FXML
  private void oneStepButtonAction() {
    stopSimulation();
    updateGame();
  }

  @FXML
  private void playToggleButtonAction() {
    startSimulation();
  }

  @FXML
  private void pauseToggleButtonAction() {
    stopSimulation();
  }

  private void startSimulation() {
    timeline.play();
  }

  private void stopSimulation() {
    timeline.stop();
  }

  private void updateGame() {
    board.updateToNextGeneration();
    grid.repaint();
    updateGenerationLabel();
  }

  private void updateGenerationLabel() {
    generationNumberLabel.setText(String.valueOf(board.stepNumber()));
  }
}
