package model;

import util.Position;

import java.util.*;

public class CloudStrategy implements Strategy {
    private List<Position> cloudPositions;
    private final Set<Position> firePositions;
    private final Map<Position, List<Position>> neighbors;
    private final Random random = new Random();
    private final int maxRows;
    private final int maxCols;

    public CloudStrategy(List<Position> cloudPositions, Set<Position> firePositions, Map<Position, List<Position>> neighbors, int maxRows, int maxCols) {

        this.cloudPositions = cloudPositions;
        this.firePositions = firePositions;
        this.neighbors = neighbors;
        this.maxRows = maxRows;
        this.maxCols = maxCols;
    }

    @Override
    public List<Position> update() {
        List<Position> modifiedCloudPositions = new ArrayList<>();
        List<Position> newCloudPositions = new ArrayList<>();

        for (Position cloudPosition : cloudPositions) {
            Position newCloudPosition = randomPos(cloudPosition);
            newCloudPositions.add(newCloudPosition);
            modifiedCloudPositions.add(cloudPosition);
            modifiedCloudPositions.add(newCloudPosition);
            extinguishFiures(newCloudPosition, modifiedCloudPositions);
        }
        cloudPositions = newCloudPositions;
        return modifiedCloudPositions;
    }

    @Override
    public List<Position> getModelPositions() {
        return cloudPositions;
    }

    @Override
    public Set<Position> getFirePositions() {
        return firePositions;
    }
    private void extinguishFiures(Position position, List<Position> modifiedCloudPositions) {
        List<Position> positions = new ArrayList<>(neighbors.getOrDefault(position, Collections.emptyList()));
        positions.add(position);
        List<Position> firePos = positions.stream().filter(firePositions :: contains).toList();
        firePositions.removeAll(firePos);
        modifiedCloudPositions.addAll(firePos);
    }
    private Position randomPos(Position cloudPosition) {
        int row = random.nextInt(3)-1;
        int col = random.nextInt(3)-1;
        int newRow = cloudPosition.getRow() + row;
        int newCol = cloudPosition.getCol() + col;
        if (newRow>=0 && newRow<maxRows && newCol>=0 && newCol<maxCols) {
            return new Position(newRow, newCol);
        }
        return cloudPosition;
    }
}
