package model;

import util.Position;
import util.TargetStrategy;

import java.util.*;

public class FireTruckmovmentStrategy implements Strategy {
    private final TargetStrategy targetStrategy = new TargetStrategy();
    private List<Position> fireTruckPositions;
    private final Set<Position> firePositions;
    private final Map<Position, List<Position>> neighbors;

    public FireTruckmovmentStrategy(List<Position> firefighterPositions, Set<Position> firePositions, Map<Position, List<Position>> neighbors) {
        this.fireTruckPositions = firefighterPositions;
        this.firePositions = firePositions;
        this.neighbors = neighbors;
    }

    @Override
    public List<Position> update() {
        List<Position> modifiedPositions = new ArrayList<>();
        List<Position> newFirefTruckPositions = new ArrayList<>();

        for (Position fireTruckPosition : fireTruckPositions) {
            Position firstStep = nextValidPosition(fireTruckPosition ); // 1er etape
            extinguishNearbyFires(firstStep, modifiedPositions);

            Position secondStep = nextValidPosition(firstStep);          // 2eme etape
            extinguishNearbyFires(secondStep, modifiedPositions);

            newFirefTruckPositions.add(secondStep);
            modifiedPositions.add(fireTruckPosition );
            modifiedPositions.add(firstStep);
            modifiedPositions.add(secondStep);
        }

        fireTruckPositions = newFirefTruckPositions;
        return modifiedPositions;
    }

    private Position nextValidPosition(Position currentPosition) {
        return targetStrategy.neighborClosestToFire(currentPosition, firePositions, neighbors);
    }

    private void extinguishNearbyFires(Position position, List<Position> modifiedPositions) {
        List<Position> firesToExtinguish = new ArrayList<>(neighbors.getOrDefault(position, List.of()));
        firesToExtinguish.add(position);
        firesToExtinguish.removeIf(pos -> !firePositions.contains(pos));

        extinguishAll(firesToExtinguish);
        modifiedPositions.addAll(firesToExtinguish);
    }

    private void extinguishAll(List<Position> positions) {
        firePositions.removeAll(positions);
    }

    @Override
    public List<Position> getModelPositions() {
        return fireTruckPositions;
    }

    @Override
    public Set<Position> getFirePositions() {
        return firePositions;
    }
}