package model;

import util.Position;
import javafx.util.Pair;
import java.util.*;



public class FirefighterBoard implements Board<List<ModelElement>> {
    private final int columnCount;
    private final int rowCount;
    private final int initialFireCount;
    private final int initialFirefighterCount;
    private final int initialRoadCount;
    private final int initialcloudCount;
    private final int initialmountainsCount;
    private final int initialFireTruckCount;
    private final int initialrocksCount;

    private List<Position> firefighterPositions;
    private Set<Position> firePositions;
    private final Set<Position> roadPositions;
    private final Map<Position, List<Position>> neighbors;
    private final Position[][] positions;
    private List<Position> cloudPositions;
    private  Set<Position> moutainsPositions;
    private  Set<Position> rocksPositions;
    private  List<Position> fireTruckPositions;

    private int step = 0;

    // Constructor for FirefighterBoard
    public FirefighterBoard(int columnCount, int rowCount, int initialFireCount, int initialFirefighterCount, int initialRoadCount,
                            int initialcloudCount, int initialmountainsCount, int initialrocksCount,int initialFireTruckCount ) {
        this.columnCount = columnCount;
        this.rowCount = rowCount;
        this.initialFireCount = initialFireCount;
        this.initialFirefighterCount = initialFirefighterCount;
        this.initialRoadCount = initialRoadCount;
        this.initialcloudCount = initialcloudCount;
        this.initialmountainsCount = initialmountainsCount;
        this.initialrocksCount = initialrocksCount;
        this.initialFireTruckCount = initialFireTruckCount;

        this.positions = new Position[rowCount][columnCount];
        this.firefighterPositions = new ArrayList<>();
        this.firePositions = new HashSet<>();
        this.neighbors = new HashMap<>();
        this.cloudPositions = new ArrayList<>();
        this.moutainsPositions = new HashSet<>();
        this.roadPositions = new HashSet<>();
        this.rocksPositions = new HashSet<>();
        this.fireTruckPositions = new ArrayList<>();

        initializeBoard();
    }

    private void initializeBoard() {
        Random random = new Random();

        // Initialize fire positions
        for (int i = 0; i < initialFireCount; i++) {
            firePositions.add(new Position(random.nextInt(rowCount), random.nextInt(columnCount)));
        }

        // Initialize firefighter positions
        for (int i = 0; i < initialFirefighterCount; i++) {
            firefighterPositions.add(new Position(random.nextInt(rowCount), random.nextInt(columnCount)));
        }
        // Initialize road positions
        for (int i = 0; i < initialRoadCount; i++) {
            roadPositions.add(new Position(random.nextInt(rowCount), random.nextInt(columnCount)));
        }
        // Initialize road positions
        for (int i = 0; i < initialcloudCount; i++) {
            cloudPositions.add(new Position(random.nextInt(rowCount), random.nextInt(columnCount)));
        }
        // Initialize Moutains Positions
        for (int i = 0; i < initialmountainsCount; i++) {
            moutainsPositions.add(new Position(random.nextInt(rowCount), random.nextInt(columnCount)));
        }
        // Initialize rocks positions
        for (int i = 0; i < initialrocksCount; i++) {
            rocksPositions.add(new Position(random.nextInt(rowCount), random.nextInt(columnCount)));
        }
        // Initialize FireTruck Positions
        for (int i = 0; i < initialFireTruckCount; i++) {
            fireTruckPositions.add(new Position(random.nextInt(rowCount), random.nextInt(columnCount)));
        }


        // Initialize neighbors map
        for (int row = 0; row < rowCount; row++) {
            for (int col = 0; col < columnCount; col++) {
                positions[row][col] = new Position(row, col);
                neighbors.put(positions[row][col], calculateNeighbors(row, col));
            }
        }
    }

    private List<Position> calculateNeighbors(int row, int col) {
        List<Position> adjacentPositions = new ArrayList<>();
        if (row > 0) adjacentPositions.add(new Position(row - 1, col));  // Top neighbor
        if (col > 0) adjacentPositions.add(new Position(row, col - 1));  // Left neighbor
        if (row < rowCount - 1) adjacentPositions.add(new Position(row + 1, col));  // Bottom neighbor
        if (col < columnCount - 1) adjacentPositions.add(new Position(row, col + 1));  // Right neighbor
        return adjacentPositions;
    }

    @Override
    public List<ModelElement> getState(Position position) {
        List<ModelElement> result = new ArrayList<>();
        for (ModelElement element : ModelElement.values()){
            switch (element){
                case FIRE -> {
                    if (firePositions.contains(position)) {
                        result.add(ModelElement.FIRE);
                    }
                }
                case FIREFIGHTER -> {
                    if (firefighterPositions.contains(position)) {
                        result.add(ModelElement.FIREFIGHTER);
                    }
                }
                case CLOUD -> {
                    if (cloudPositions.contains(position)) {
                        result.add(ModelElement.CLOUD);
                    }
                }
                case ROAD -> {
                    if (roadPositions.contains(position)) {
                        result.add(ModelElement.ROAD);
                    }
                }
                case MOUTAIN -> {
                    if (moutainsPositions.contains(position)) {
                        result.add(ModelElement.MOUTAIN);
                    }
                }
                case ROCKS -> {
                    if (rocksPositions.contains(position)) {
                        result.add(ModelElement.ROCKS);
                    }
                }
                case FIRETRUCK -> {
                    if (fireTruckPositions.contains(position)) {
                        result.add(ModelElement.FIRETRUCK);
                    }
                }
                default ->{
                }
            }
        }
        return result;
    }

    @Override
    public int rowCount() {
        return rowCount;
    }

    @Override
    public int columnCount() {
        return columnCount;
    }

    @Override
    public int stepNumber() {
        return step;
    }

    @Override
    public void reset() {
        step = 0;
        Initializer initializer = new Initializer(columnCount,rowCount,initialFireCount,initialFirefighterCount,
                initialRoadCount,initialcloudCount, initialmountainsCount, initialFireTruckCount,initialrocksCount);

        resetPos(firePositions,initializer.getFirePositions());
        resetPos(firefighterPositions,initializer.getFirefighterPositions());
        resetPos(cloudPositions,initializer.getCloudPositions());
        resetPos(moutainsPositions,initializer.getMountainsPositions());
        resetPos(roadPositions,initializer.getRoadPositions());
        resetPos(rocksPositions,initializer.getRocksPositions());
        resetPos(fireTruckPositions,initializer.getFireTruckPositions());

        neighbors.clear();
        neighbors.putAll(initializer.getNeighbors());

        initializeBoard();
    }
    private <S> void resetPos(Collection<S> currentPositions, Collection<S> newPositions) {
        currentPositions.clear();
        currentPositions.addAll(newPositions);
  }

    @Override
    public void setState(List<ModelElement> state, Position position) {
        firePositions.remove(position);
        firefighterPositions.remove(position);
        fireTruckPositions.remove(position);

        for (ModelElement element : state) {
            switch (element) {
                case FIRE -> firePositions.add(position);
                case FIREFIGHTER -> firefighterPositions.add(position);
                case CLOUD -> cloudPositions.add(position);
                case FIRETRUCK -> fireTruckPositions.add(position);
            }
        }
    }

    @Override
    public List<Position> updateToNextGeneration() {
        List<ModelElement> elements = List.of(
                ModelElement.FIRE,
                ModelElement.FIREFIGHTER,
                ModelElement.CLOUD,
                ModelElement.FIRETRUCK
        );
        List<Position> modifiedPositions = new ArrayList<>();

        for(ModelElement element : elements) {
            Strategy strategy = switch(element) {
                case FIRE -> new SimpleFireSpreadStrategy(firePositions, neighbors,moutainsPositions,roadPositions, rocksPositions,step);
                case FIREFIGHTER -> new FirefighterMovementStrategy(firefighterPositions,firePositions,neighbors);
                case CLOUD -> new CloudStrategy(cloudPositions,firePositions,neighbors,rowCount,columnCount);
                case FIRETRUCK -> new FireTruckmovmentStrategy(fireTruckPositions,firePositions,neighbors);
                default -> throw new IllegalStateException("Type de modèle inconnu : " + element);
            };
            modifiedPositions.addAll(strategy.update());

            switch (element) {
                case FIRE -> firePositions = strategy.getFirePositions();
                case FIREFIGHTER -> firefighterPositions = strategy.getModelPositions();
                case CLOUD -> cloudPositions = strategy.getModelPositions();
                case FIRETRUCK -> fireTruckPositions = strategy.getModelPositions();
            }
        }
        step++;
        return modifiedPositions;
    }

    public List<Pair<Position, ModelElement>> getUpdatedElements() {
        List<Pair<Position, ModelElement>> updatedElements = new ArrayList<>();
        System.out.println(initialFireCount);
        // Add fire positions
        for (Position firePosition : firePositions) {
            updatedElements.add(new Pair<>(firePosition, ModelElement.FIRE));
        }
        // Add firefighter positions
        for (Position firefighterPosition : firefighterPositions) {
            updatedElements.add(new Pair<>(firefighterPosition, ModelElement.FIREFIGHTER));
        }
        for (Position cloudPosition : cloudPositions) {
            updatedElements.add(new Pair<>(cloudPosition, ModelElement.CLOUD));
        }
        for (Position roadPosition : roadPositions) {
            updatedElements.add(new Pair<>(roadPosition, ModelElement.ROAD));
        }
        for (Position moutainPosition : moutainsPositions){
            updatedElements.add(new Pair<>(moutainPosition, ModelElement.MOUTAIN));
        }
        for (Position FireTruckPosition : fireTruckPositions){
            updatedElements.add(new Pair<>(FireTruckPosition, ModelElement.FIRETRUCK));
        }
        for (Position rockPosition : rocksPositions){
            updatedElements.add(new Pair<>(rockPosition,ModelElement.ROCKS));
        }

        return updatedElements;
    }
}










