package model;

import util.Position;

import java.util.*;

import static app.SimulatorApplication.*;

public class Initializer {
    private final int columnCount;
    private final int rowCount;
    private final int initialFireCount;
    private final int initialFirefighterCount;
    private final int initialRoadCount;
    private final int initialcloudCount;
    private final int initialMoutainCount;
    private final int initialFireTruckCount;
    private final int initialRocksCount;


    private final Position[][] positions;
    private final Map<Position, List<Position>> neighbors;
    private List<Position> RoadPositions;
    private final List<Position> firefighterPositions;
    private final Set<Position> firePositions;
    private final List<Position> cloudPositions;
    private final Set<Position> mountainsPositions;
    private final List<Position> FireTruckPositions;
    private final Set<Position> rocksPositions;
    private int step = 0;

    private final Random randomGenerator = new Random();


    public Initializer(int columnCount, int rowCount, int initialFireCount, int initialFirefighterCount, int initialRoadCount,
                       int initialcloudCount, int initialMoutainCount, int initialRocksCount,int initialFireTruckCount) {
        this.columnCount = columnCount;
        this.rowCount = rowCount;
        this.initialFireCount = initialFireCount;
        this.initialFirefighterCount = initialFirefighterCount;
        this.initialRoadCount = initialRoadCount;
        this.initialcloudCount = initialcloudCount;
        this.initialMoutainCount = initialMoutainCount;
        this.initialRocksCount = initialRocksCount;
        this.initialFireTruckCount = initialFireTruckCount;

        this.positions = new Position[rowCount][columnCount];
        this.neighbors = new HashMap<>();
        this.firePositions = new HashSet<>();
        this.firefighterPositions = new ArrayList<>();
        this.RoadPositions = new ArrayList<>();
        this.cloudPositions = new ArrayList<>();
        this.mountainsPositions = new HashSet<>();
        this.FireTruckPositions = new ArrayList<>();
        this.rocksPositions = new HashSet<>();


    }
    private void initializePositions() {
        for (int row = 0; row < rowCount; row++) {
            for (int column = 0; column < columnCount; column++) {
                positions[row][column] = new Position(row, column);
            }
        }
    }
    private void initializeNeighbors() {
        for (int row = 0; row < rowCount; row++) {
            for (int column = 0; column < columnCount; column++) {
                List<Position> adjacentPositions = new ArrayList<>();
                if (row > 0) adjacentPositions.add(new Position(row - 1, column));  // Top neighbor
                if (column > 0) adjacentPositions.add(new Position(row, column - 1));  // Left neighbor
                if (row < rowCount - 1) adjacentPositions.add(new Position(row + 1, column));  // Bottom neighbor
                if (column < columnCount - 1) adjacentPositions.add(new Position(row, column + 1));  // Right neighbor
                neighbors.put(positions[row][column], adjacentPositions);
            }
        }
    }
    private void initializeElements() {
        initializeFirePositions();
        initializeFirefighters();
        initializeRoadPositions();
        initializeCloudsPositions();
        initializeMoutainsPositions();
        initializeFireTruckPositions();
        initializeRocksPositions();
    initializeFirePositions();
    initializeFirefighters();
    }
    private void initializeFirePositions() {
        addRandomPositions(firePositions, initialFireCount);
    }
    private void initializeFirefighters() {
            addRandomPositions(firefighterPositions, initialFirefighterCount);
    }
    private void initializeRoadPositions() {
        addRandomPositions(RoadPositions, initialRoadCount);}
    private void initializeCloudsPositions() {
        addRandomPositions(cloudPositions, initialcloudCount);
    }
    private void initializeFireTruckPositions() {
        addRandomPositions(FireTruckPositions, initialFireTruckCount);
    }

    private void initializeMoutainsPositions() {addRandomPositions(mountainsPositions, initialMoutainCount);}
    private void initializeRocksPositions() {addRandomPositions(rocksPositions, initialRocksCount);}


    private void addRandomPositions(Collection<Position> targetCollection, int count) {
        while (targetCollection.size() < count) {
            Position randomPosition = randomPosition();
            if (isValid(randomPosition)) {
                targetCollection.add(randomPosition);
            }
        }
    }
    private boolean isValid(Position targetPosition) {
        return !firePositions.contains(targetPosition);
    }
    private Position randomPosition() {
        return new Position(randomGenerator.nextInt(rowCount), randomGenerator.nextInt(columnCount));
    }

    public int getColumnCount() {
        return columnCount;
    }

    public int getRowCount() {
        return rowCount;
    }

    public int getInitialFireCount() {
        return initialFireCount;
    }

    public int getInitialFirefighterCount() {
        return initialFirefighterCount;
    }

    public Position[][] getPositions() {
        return positions;
    }

    public Map<Position, List<Position>> getNeighbors() {
        return neighbors;
    }

    public List<Position> getFirefighterPositions() {
        return firefighterPositions;
    }

    public Set<Position> getFirePositions() {
        return firePositions;
    }
    public List<Position> getRoadPositions() {
        return RoadPositions;
    }
    public List<Position> getCloudPositions() {
        return cloudPositions;
    }
    public Set<Position> getMountainsPositions() {
        return mountainsPositions;
    }
    public List<Position> getFireTruckPositions() {
        return FireTruckPositions;
    }
    public Set<Position> getRocksPositions() {
        return rocksPositions;
    }
    public int getStep() {return step;}

}
